package jp.sf.grizzly.hatena.pipeline.valve;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.List;

import jp.sf.grizzly.pipeline.GrizzlyPipeline;
import jp.sf.grizzly.pipeline.PipelineException;
import jp.sf.grizzly.storage.StreamStorage;
import jp.sf.grizzly.storage.StreamStorageException;
import jp.sf.grizzly.storage.impl.FileStreamStorageImpl;
import junit.framework.TestCase;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class ConvertTimeCaptionValveTest extends TestCase
{
	private static final String CONVERT_STRING = "*1149410499*お昼に書きます";

	private static final String CONVERT_RESULT = "<p><span class=\"sanchor\">■</span> <span class=\"h3\">お昼に書きます <span class=\"timestamp\">17:41</span></span></p>";

	/**
	 * Logger for this class
	 */
	private static final Log log = LogFactory
			.getLog(ConvertTimeCaptionValveTest.class);

	GrizzlyPipeline pipeline = null;

	List valveList = new ArrayList();

	protected void setUp() throws Exception
	{
		valveList.add(new ConvertTimeCaptionValve());
	}

	/**
	 * Rigourous Test :-)
	 */
	public void testInvoke()
	{
		StringBuffer buffer = new StringBuffer();
		try
		{
			pipeline = new GrizzlyPipeline("", valveList);
			StreamStorage storage = new FileStreamStorageImpl(
					new ByteArrayInputStream(CONVERT_STRING.getBytes("UTF-8")),
					"UTF-8");
			pipeline.invoke(storage);

			BufferedReader r = null;
			try
			{
				r = new BufferedReader(new InputStreamReader(storage
						.getResultInputStream(), storage.getEncoding()));
				String l = null;
				while ((l = r.readLine()) != null)
				{
					buffer.append(l);
				}
			}
			catch (UnsupportedEncodingException e)
			{
				log.warn("Unsupported Encoding. ", e);
			}
			catch (StreamStorageException e)
			{
				log.warn("Stream Storage Exception. ", e);
			}
			catch (IOException e)
			{
				log.error("I/O Exception. ", e);
			}
			finally
			{
				if(r != null)
				{
					try
					{
						r.close();
					}
					catch (IOException e)
					{
					}
				}
			}
			storage.destroy();
		}
		catch (StreamStorageException e)
		{
			log.error("StreamStorage Exception.", e);
		}
		catch (PipelineException e)
		{
			log.error("Pipeline Exception.", e);
		}
		catch (Exception e)
		{
			log.error("Exception.", e);
		}

		assertEquals(CONVERT_RESULT, buffer.toString());
	}

}
